
#include "array_wrapper.h"
#include "range_wrapper.h"

using namespace CNORXZ;

int PyCxRange_init(PyCxRange* self, PyObject* args, PyObject* kwds)
{

    //!!!
    return 0;
}

void PyCxRange_dealloc(PyCxRange* self)
{
    delete self->ptrObj;
    Py_TYPE(self)->tp_free(self);
}

PyObject* PyCxRange_size(PyCxRange* self)
{
    if(not self->ptrObj->_init()){
	const SizeT retval = 0;
	return Py_BuildValue("k", retval);
    }
    const SizeT retval = self->ptrObj->size();
    return Py_BuildValue("k", retval);
}

PyObject* PyCxRange_dim(PyCxRange* self)
{
    if(not self->ptrObj->_init()){
	const SizeT retval = 0;
	return Py_BuildValue("k", retval);
    }
    const SizeT retval = self->ptrObj->dim();
    return Py_BuildValue("k", retval);
}

PyObject* PyCxRange_sub(PyCxRange* self, PyObject* args, PyObject* kwds)
{
    static char* kwlist[] = { "pos", NULL };
    SizeT pos = 0;
    
    if(not PyArg_ParseTupleAndKeywords(args, kwds, "k|", kwlist, &pos)){
	return NULL;
    }

    SizeT dim = self->ptrObj->dim();
    if(pos >= dim){
	return PyErr_Format(PyExc_RuntimeError, "requested sub-range position (%d) exceeds range dimension (%d)", pos, dim);
    }

    // entire set of sub-ranges:
    /*
    PyCxArrayB* o;
    o = PyObject_New(PyCxArrayB, &PyCxArrayBType);
    o->ptrObj = new ArrayWrapper<RangePtr>( std::make_shared<MArray<RangePtr>>(self->ptrObj->sub()) );
    return Py_BuildValue("O", o);
    */
    
    const RangePtr r = self->ptrObj->sub(pos);
    PyCxRange* o = PyObject_New(PyCxRange, &PyCxRangeType);
    o->ptrObj = new RangeWrapper(r);
    return Py_BuildValue("O", o);
}

PyObject* PyCxRange_stringMeta(PyCxRange* self, PyObject* args, PyObject* kwds)
{
    const SizeT size = self->ptrObj->size();
    PyObject* out = PyList_New(size);
    for(SizeT i = 0; i != size; ++i){
	PyObject* s = Py_BuildValue("s", self->ptrObj->stringMeta(i).c_str() );
	PyList_SetItem(out, i, s);
    }
    return out;
}

PyMethodDef PyCxRange_methods[] = {
    { "size", (PyCFunction) PyCxRange_size, METH_VARARGS, "return range size" },
    { "dim", (PyCFunction) PyCxRange_dim, METH_VARARGS, "return range dimension" },
    { "sub", (PyCFunction) PyCxRange_sub, METH_VARARGS|METH_KEYWORDS, "return sub-range for given dimension" },
    { "stringMeta", (PyCFunction) PyCxRange_stringMeta, METH_VARARGS,
      "return string meta data for given range position" },
    { NULL }
};

PyTypeObject PyCxRangeType = { PyVarObject_HEAD_INIT(NULL,0) "cnorxz.Range" };

PyTypeObject* PyCxRangeType_init()
{
    PyCxRangeType.tp_new = PyType_GenericNew;
    PyCxRangeType.tp_basicsize = sizeof(PyCxRange);
    PyCxRangeType.tp_dealloc = (destructor) PyCxRange_dealloc;
    PyCxRangeType.tp_flags = Py_TPFLAGS_DEFAULT;
    PyCxRangeType.tp_doc = "cnorxz Range wrapper";
    PyCxRangeType.tp_methods = PyCxRange_methods;
    PyCxRangeType.tp_init = (initproc) PyCxRange_init;

    if(PyType_Ready(&PyCxRangeType) < 0){
	return NULL;
    }
    else {
	return &PyCxRangeType;
    }
}
